import pandas as pd
import numpy as np
import os

class Star(object):
    """
    A simulated Star that holds all the information needed to plot an example spectra.
    
    """

    def __init__(self, effective_temperature, surface_gravity, metallicity):
        """
        Function that initializes our star and the three selected parameters needed to determined
        which spectra is generated
        """

        self.effective_temperature = effective_temperature
        self.surface_gravity = surface_gravity
        self.metallicity = metallicity

    def select_spectra(self):
        """
        This function selects a given spectra to plot given the instatiated values for the
        star in question
        """
        spec_list = pd.read_csv('obafgkm/star_types.csv')
        subselect = spec_list[(spec_list['effective_temp']==self.effective_temperature) & (spec_list['surface_gravity']==self.surface_gravity) & (spec_list['metallicity']==self.metallicity)]
        filename = subselect['filepath'].values[0]
        unnormalized = np.loadtxt(os.path.join('obafgkm/unnormalized_spectra', filename), skiprows=0)
        normalized = np.loadtxt(os.path.join('obafgkm/normalized_spectra', filename), skiprows=0)
        parameters = (self.effective_temperature, self.surface_gravity, self.metallicity)
        return (unnormalized, normalized, parameters, filename)


import numpy as np
from main import Star
from plot import plotter
types=[3000, 4000, 5000, 6000, 7000, 8000, 0.0, 1.0, 2.0, 3.0, 4.0, 5.0]
metals = ['supersolar', 'solar', 'subsolar']

while True:
    print("Select effective temperature class: (options are: '3000', '4000', '5000', '6000', '7000', '8000')")
    try:
        input_teff = int(input())
        if input_teff not in types:
            raise ValueError('stuff is not in content')
    except:
        print("Invalid input. Please enter a valid spectral type.")
        continue
    else:
        break
while True:
    print("Select surface gravity: (options include log(g) [dex] values of: '0.0', '1.0', '2.0', '3.0', '4.0', '5.0')")
    try:
        input_logg = float(input())
        if input_logg not in types:
            raise ValueError('stuff is not in content')
    except ValueError:
        print("Invalid input. Please enter a valid metallicity.")
        continue
    else:
        break
while True:
    print("Select [M/H]: (options include 'supersolar' ([M/H]=0.5), 'solar' ([M/H]=0.0), or 'subsolar' ([M/H]=-2.0))")
    try:
        input_metallicity = str(input())
        if input_metallicity not in metals:
            raise ValueError('stuff is not in content')
    except ValueError:
        print("Invalid input. Please enter a valid metallicity.")
        continue
    else:
        break
plot_star = Star(input_teff, input_logg, input_metallicity)

stellar_info = plot_star.select_spectra() # four value tuple

plotter(stellar_info, input("Would you like to save the plot? (y/n): ").lower() == 'y')


from astropy.table import Table
import numpy as np
import matplotlib.pyplot as plt
import os
import pathlib

os.chdir(pathlib.Path.cwd())
def set_rcparams():
    tab = Table.read('obafgkm/rcparams.txt', format='csv')
    for i in range(len(tab)):
        try:
            plt.rcParams[tab['key'][i]] = float(tab['val'][i])
        except ValueError:
            plt.rcParams[tab['key'][i]] = str(tab['val'][i])
    return

def plotter(spectra_tuple, save=False):
    """
    Plotting function to display a spectra graphically
    
    """
    set_rcparams()
    unnormalized_spectra, normalized_spectra, parameters, filepath = spectra_tuple
    wav_unnormed, flux_unnormed = (unnormalized_spectra[:,0], unnormalized_spectra[:,1])
    wav_normed, flux_normed = (normalized_spectra[:,0], normalized_spectra[:,1])

    (teff, logg, m_h) = [parameters[i] for i in range(3)]

    fig, [ax1, ax2]  = plt.subplots(2, 1, figsize=(20, 14), layout='constrained')
    
    ax1.plot(wav_unnormed, flux_unnormed, color='blue', lw=0.5)
    ax2.plot(wav_normed, flux_normed, color='r', lw=0.5)

    fig.suptitle(r"Spectrum of T$_{\rm{eff}}$=" + str(teff) + ", $\log$(g)=" + str(logg) + ", [M/H]=" + str(m_h), weight='bold')
    fig.supxlabel(r'Wavelength ($\rm{\AA}$)')
    
    ax1.set_ylabel('Flux [erg/s/cm$^{5}$]')
    ax2.set_ylabel('Normalized Flux')
    ax1.set_yscale('log')
    [ax.grid(True) for ax in [ax1, ax2]]
    [ax.axvline(3968.47, c='orange', lw=0.5, label='Ca H') for ax in [ax1, ax2]]
    [ax.axvline(3933.66, c='orange', lw=0.5, label='Ca K') for ax in [ax1, ax2]]
    [ax.axvline(4861.34, c='k', lw=0.5, linestyle='-.', label=r'H $\beta$') for ax in [ax1, ax2]]
    [ax.axvline(5889.95, c='purple', lw=0.5, linestyle='--', label=r'Na D$_{2}$') for ax in [ax1, ax2]]
    [ax.axvline(5895.92, c='purple', lw=0.5, linestyle='--', label=r'Na D$_{1}$',) for ax in [ax1, ax2]]
    [ax.axvline(6562.81, c='k', lw=0.5, linestyle=':', label=r'H $\alpha$') for ax in [ax1, ax2]]
    [ax.legend(ncols=3, fontsize=12) for ax in [ax1, ax2]]
    if save:
        if not os.path.exists('obafgkm/plots'):
            os.makedirs('obafgkm/plots')
        if(os.path.exists(os.path.join('obafgkm/plots', filepath.split('.txt')[0] + '_spectrum.png'))):
            print(f"This spectrum has already been saved! Look in plots/{filepath.split('.txt')[0]}_spectrum.png")
        else:
            plt.savefig(os.path.join('obafgkm/plots', filepath.split('.txt')[0] + '_spectrum.png'), dpi=300)
    plt.show()